﻿using System;
using SubSpec;
using Xunit;
using System.Xml;
using TestUtility;
using SubSpec.Expression;

public class SpecificationExpressionExecution : AcceptanceTest
{
    [Fact( Skip = "Todo" )]
    public void OneTestPerObservation()
    {
        string code =
            @"
                using System;
				using System.Text;
                using Xunit;
				using SubSpec;
				using SubSpec.Expression;

                public class MockTestClass
                {
					[Specification]
					public void ExpressionSyntax()
					{
						var spec = from sut in SpecificationExpression.Begin(() => new StringBuilder())
								   where sut.Append(""x"")
								   select new Observations() 
									{ 
										() => Assert.NotNull(sut), 
										() => Assert.Equal(""y"", sut.ToString()) 
									};
					}
                }
            ";

        XmlNode assemblyNode = ExecuteWithReferences( code, typeof( SpecificationAttribute ).Assembly.Location );

        var first = ResultXmlUtility.GetResult( assemblyNode, 0 );
        var second = ResultXmlUtility.GetResult( assemblyNode, 1 );

        ResultXmlUtility.AssertAttribute( first, "result", "Pass" );
        ResultXmlUtility.AssertAttribute( second, "result", "Fail" );
    }

    [Fact( Skip = "Todo" )]
    public void WhereClauseIsOptional()
    {
        string code =
            @"
                using System;
				using System.Text;
                using Xunit;
				using SubSpec;
				using SubSpec.Expression;

                public class MockTestClass
                {
					[Specification]
					public void SpecExpWithoutWhere()
					{
						var spec = from sut in SpecificationExpression.Begin(() => new StringBuilder())
								   select new Observations() 
									{ 
										() => Assert.NotNull(sut)
									};
					}
                }
            ";

        XmlNode assemblyNode = ExecuteWithReferences( code, typeof( SpecificationAttribute ).Assembly.Location );

        var first = ResultXmlUtility.GetResult( assemblyNode, 0 );

        ResultXmlUtility.AssertAttribute( first, "result", "Pass" );
    }

    [Fact( Skip = "Todo" )]
    public void TestNamesAreDerivedFromSpec()
    {
        string code =
            @"
                using System;
				using System.Text;
                using Xunit;
				using SubSpec;
				using SubSpec.Expression;

                public class MockTestClass
                {
					[Specification]
					public void ACoolSpecification()
					{
						var spec = from sut in SpecificationExpression.Begin(() => string.Empty)
								   select new Observations() 
									{ 
										() => Assert.True(true)
									};
					}
                }
            ";

        XmlNode assemblyNode = ExecuteWithReferences( code, typeof( SpecificationAttribute ).Assembly.Location );

        var first = ResultXmlUtility.GetResult( assemblyNode, 0 );

        ResultXmlUtility.AssertAttribute( first, "result", "Pass" );
        ResultXmlUtility.AssertAttribute( first, "name", "ACoolSpecification, " );
    }
}
