﻿using System;
using SubSpec;
using Xunit;
using System.Xml;
using TestUtility;
using Xunit.Extensions;
using System.Linq;

public class ThesisIntegration : AcceptanceTest
{
    [Fact]
    public void OneSpecificationPerThesisDataItem()
    {
        string code =
            @"
                using System;
                using Xunit;
				using Xunit.Extensions;
				using SubSpec;

                public class MockTestClass
                {
						[Thesis]
						[InlineData(1, ""A"")]
						[InlineData(2, ""B"")]
						public void OneSpecificationPerThesisDataItem(int i, string s)
						{
							i.ToString().Context(() => { });

							(s + ""= A?"").Assert(() => { Assert.Equal(""A"", s); });
							(s + ""= B?"").Assert(() => { Assert.Equal(""B"", s); });
						}
                }
            ";

        XmlNode assemblyNode = ExecuteWithReferences( code, typeof( SpecificationAttribute ).Assembly.Location, typeof( TheoryAttribute ).Assembly.Location );

        // Parameter "A"
        SubSpecResultUtility.VerifyPassed( assemblyNode.AssertionResults().First() );
        SubSpecResultUtility.VerifyFailed( assemblyNode.AssertionResults().Second() );

        // Parameter "B"
        SubSpecResultUtility.VerifyFailed( assemblyNode.AssertionResults().ElementAt( 2 ) );
        SubSpecResultUtility.VerifyPassed( assemblyNode.AssertionResults().ElementAt( 3 ) );
    }

    [Fact]
    public void ThesisHandlesExceptionsInSpecificationSetup()
    {
        string code =
            @"
                using System;
                using Xunit;
				using Xunit.Extensions;
				using SubSpec;

                public class MockTestClass
                {
						[Thesis]
						[InlineData(""A"")]
						[InlineData(""B"")]
						public void Throws(string s)
						{
							throw new Exception(""Error in Specification"");
						}
                }
            ";

        XmlNode assemblyNode = ExecuteWithReferences( code, typeof( SpecificationAttribute ).Assembly.Location, typeof( TheoryAttribute ).Assembly.Location );

        var first = ResultXmlUtility.GetResult( assemblyNode, 0 );
        var second = ResultXmlUtility.GetResult( assemblyNode, 1 );

        SubSpecResultUtility.VerifyFailedWith( first, "System.Exception : Error in Specification" );
        SubSpecResultUtility.VerifyFailedWith( second, "System.Exception : Error in Specification" );
    }

    public void ThesisHandlesNoDataInTheory()
    {
        string code =
            @"
                using System;
                using Xunit;
				using Xunit.Extensions;
				using SubSpec;

                public class MockTestClass
                {
						[Thesis]
						public void Spec()
						{
							"""".Context(() => { });
							"""".Observation(() => { });
						}
                }
            ";

        XmlNode assemblyNode = ExecuteWithReferences( code, typeof( SpecificationAttribute ).Assembly.Location, typeof( TheoryAttribute ).Assembly.Location );

        var first = ResultXmlUtility.GetResult( assemblyNode, 0 );

        SubSpecResultUtility.VerifyFailedWith( first, "System.InvalidOperationException : No data found for MockTestClass.Spec" );
    }
}
