﻿using System;
using Xunit;
using System.Threading;
using SubSpec;

public class ThreadIsolationFacts
{
    [Fact]
    public void CanSetUpSpecificationConcurrently()
    {
        VerifyConcurrentExecution( SetUpSpecification );
    }

    private void VerifyConcurrentExecution( ThreadStart action )
    {
        Assert.DoesNotThrow( () =>
        {
            // We could use the Task API here but we want to be explicit about getting two concurrent, physical threads
            Thread a = new Thread( action );
            Thread b = new Thread( action );

            a.Start();
            b.Start();

            a.Join();
            b.Join();
        } );
    }

    private void SetUpSpecification()
    {
        "".Context( () => { } );
        "".Do( () => { } );
        "".Assert( () => { } );
    }

    [Fact]
    public void DoEnsuresThreadStatic()
    {
        VerifyConcurrentExecution( () => "".Do( () => { } ) );
    }

    [Fact]
    public void AssertEnsuresThreadStatic()
    {
        VerifyConcurrentExecution( () => "".Assert( () => { } ) );
    }

    [Fact]
    public void ObservationEnsuresThreadStatic()
    {
        VerifyConcurrentExecution( () => "".Observation( () => { } ) );
    }

    [Fact]
    public void TodoEnsuresThreadStatic()
    {
        VerifyConcurrentExecution( () => "".Todo( () => { } ) );
    }

    [Fact]
    public void CanEnumerateTestCommandsOfEmptySpecificationConcurrently()
    {
        VerifyConcurrentExecution( () => SpecificationContext.ToTestCommands( null ) );
    }
}
